@@ -56,9 +56,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -72,10 +72,10 @@ CSG_Table * SG_Create_Table(const CSG_Table &Table)
 {
 	switch( Table.Get_ObjectType() )
 	{
-	case SG_DATAOBJECT_TYPE_Table:
+	case SG_DATAOBJECT_TYPE_Table     :
 		return( new CSG_Table(Table) );
 
-	case SG_DATAOBJECT_TYPE_Shapes:
+	case SG_DATAOBJECT_TYPE_Shapes    :
 	case SG_DATAOBJECT_TYPE_PointCloud:
 		return( SG_Create_Shapes(*((CSG_Shapes *)&Table)) );
 
@@ -91,10 +91,10 @@ CSG_Table * SG_Create_Table(CSG_Table *pTemplate)
 	{
 		switch( pTemplate->Get_ObjectType() )
 		{
-		case SG_DATAOBJECT_TYPE_Table:
+		case SG_DATAOBJECT_TYPE_Table     :
 			return( new CSG_Table(pTemplate) );
 
-		case SG_DATAOBJECT_TYPE_Shapes:
+		case SG_DATAOBJECT_TYPE_Shapes    :
 		case SG_DATAOBJECT_TYPE_PointCloud:
 			return( SG_Create_Shapes((CSG_Shapes *)pTemplate) );
 
@@ -111,7 +111,14 @@ CSG_Table * SG_Create_Table(const char       *File, TSG_Table_File_Type Format,
 CSG_Table * SG_Create_Table(const wchar_t    *File, TSG_Table_File_Type Format, int Encoding) { return( SG_Create_Table(CSG_String(File), Format, Encoding) ); }
 CSG_Table * SG_Create_Table(const CSG_String &File, TSG_Table_File_Type Format, int Encoding)
 {
-	return( new CSG_Table(File, Format, Encoding) );
+	CSG_Table *pTable = new CSG_Table();
+
+	if( pTable->Create(File, Format, Encoding) )
+	{
+		return( pTable );
+	}
+
+	delete(pTable); return( NULL );
 }
 
 //---------------------------------------------------------
@@ -119,14 +126,21 @@ CSG_Table * SG_Create_Table(const char       *File, TSG_Table_File_Type Format,
 CSG_Table * SG_Create_Table(const wchar_t    *File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding) { return( SG_Create_Table(CSG_String(File), Format, Separator, Encoding) ); }
 CSG_Table * SG_Create_Table(const CSG_String &File, TSG_Table_File_Type Format, const SG_Char Separator, int Encoding)
 {
-	return( new CSG_Table(File, Format, Encoding) );
+	CSG_Table *pTable = new CSG_Table();
+
+	if( pTable->Create(File, Format, Separator, Encoding) )
+	{
+		return( pTable );
+	}
+
+	delete(pTable); return( NULL );
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -183,48 +197,27 @@ bool CSG_Table::Create(const CSG_String &File, TSG_Table_File_Type Format, int E
 	{
 		CSG_String s(File);
 
-		s = s.AfterFirst(':'); CSG_String Host  (s.BeforeFirst(':'));
-		s = s.AfterFirst(':'); CSG_String Port  (s.BeforeFirst(':'));
-		s = s.AfterFirst(':'); CSG_String DBName(s.BeforeFirst(':'));
-		s = s.AfterFirst(':'); CSG_String Table (s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String Host (s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String Port (s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String DBase(s.BeforeFirst(':'));
+		s = s.AfterFirst(':'); CSG_String Table(s.BeforeFirst(':'));
 
-		CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 0, true); // CGet_Connections
+		CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 12, true); // CPGIS_Table_Load
 
-		if(	pTool != NULL )
+		if( pTool )
 		{
 			SG_UI_ProgressAndMsg_Lock(true);
 
-			//---------------------------------------------
-			CSG_Table Connections; CSG_String Connection(DBName + " [" + Host + ":" + Port + "]");
+			CSG_String Connection(DBase + " [" + Host + ":" + Port + "]");
 
-			pTool->Set_Manager(NULL); pTool->On_Before_Execution();
-
-			if( SG_TOOL_PARAMETER_SET("CONNECTIONS", &Connections) && pTool->Execute() ) // CGet_Connections
-			{
-				for(int i=0; !bResult && i<Connections.Get_Count(); i++)
-				{
-					if( !Connection.Cmp(Connections[i].asString(0)) )
-					{
-						bResult = true;
-					}
-				}
-			}
+			bResult = pTool->Set_Manager(NULL) && pTool->On_Before_Execution()
+			       && pTool->Set_Parameter("CONNECTION", Connection)
+			       && pTool->Set_Parameter("DB_TABLE"  , Table     )
+			       && pTool->Set_Parameter("TABLE"     , this      )
+			       && pTool->Execute();
 
 			SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
 
-			//---------------------------------------------
-			if( bResult && (bResult = (pTool = SG_Get_Tool_Library_Manager().Create_Tool("db_pgsql", 12, true)) != NULL) == true ) // CPGIS_Table_Load
-			{
-				pTool->Set_Manager(NULL); pTool->On_Before_Execution();
-
-				bResult =  SG_TOOL_PARAMETER_SET("CONNECTION", Connection)
-						&& SG_TOOL_PARAMETER_SET("DB_TABLE"  , Table     )
-						&& SG_TOOL_PARAMETER_SET("TABLE"     , this      )
-						&& pTool->Execute();
-
-				SG_Get_Tool_Library_Manager().Delete_Tool(pTool);
-			}
-
 			SG_UI_ProgressAndMsg_Lock(false);
 		}
 	}
@@ -295,7 +288,7 @@ bool CSG_Table::Create(const CSG_Table *pTemplate)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -319,7 +312,7 @@ void CSG_Table::_On_Construction(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -360,9 +353,9 @@ bool CSG_Table::Destroy(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Assign							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -374,13 +367,20 @@ CSG_Table & CSG_Table::operator = (const CSG_Table &Table)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Assign(CSG_Data_Object *pObject)
+bool CSG_Table::Assign(CSG_Data_Object *pObject, bool bProgress)
 {
-	if( CSG_Data_Object::Assign(pObject) && pObject->asTable(true) && Create(pObject->asTable(true)) )
+	if( pObject && pObject->asTable(true) && CSG_Data_Object::Assign(pObject) )
 	{
 		CSG_Table *pTable = pObject->asTable(true);
 
-		for(sLong i=0; i<pTable->Get_Count(); i++)
+		m_Encoding = pTable->m_Encoding;
+
+		for(int i=0; i<pTable->Get_Field_Count(); i++)
+		{
+			Add_Field(pTable->Get_Field_Name(i), pTable->Get_Field_Type(i));
+		}
+
+		for(sLong i=0; i<pTable->Get_Count() && (!bProgress || SG_UI_Process_Set_Progress(i, pTable->Get_Count())); i++)
 		{
 			Add_Record(pTable->Get_Record(i));
 		}
@@ -421,9 +421,9 @@ bool CSG_Table::Assign_Values(const SG_Char *FileName)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Checks							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -431,31 +431,15 @@ bool CSG_Table::is_Compatible(const CSG_Table &Table, bool bExactMatch) const
 {
 	if( Get_Field_Count() == Table.Get_Field_Count() )
 	{
-		for(int i=0; i<Get_Field_Count(); i++)
+		if( bExactMatch )
 		{
-			if( bExactMatch )
+			for(int i=0; i<Get_Field_Count(); i++)
 			{
 				if( Get_Field_Type(i) != Table.Get_Field_Type(i) )
 				{
 					return( false );
 				}
 			}
-			else switch( Get_Field_Type(i) )
-			{
-			case SG_DATATYPE_String:
-			//	if( Table.Get_Field_Type(i) != SG_DATATYPE_String )
-			//	{
-			//		return( false );
-			//	}
-				break;
-
-			default:
-				if( Table.Get_Field_Type(i) == SG_DATATYPE_String )
-				{
-					return( false );
-				}
-				break;
-			}
 		}
 
 		return( true );
@@ -472,9 +456,9 @@ bool CSG_Table::is_Compatible(CSG_Table *pTable, bool bExactMatch) const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Fields							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -604,7 +588,7 @@ bool CSG_Table::Mov_Field(int iField, int Position)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -623,66 +607,110 @@ bool CSG_Table::Set_Field_Name(int iField, const SG_Char *Name)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Set_Field_Type(int iField, TSG_Data_Type Type)
+bool CSG_Table::Set_Field_Type(int Field, TSG_Data_Type Type)
 {
-	if( iField < 0 || iField >= m_nFields )
+	if( Field < 0 || Field >= m_nFields )
 	{
 		return( false );
 	}
 
-	if( m_Field_Type[iField] == Type )
+	if( m_Field_Type[Field] == Type )
 	{
 		return( true );
 	}
 
-	m_Field_Type[iField] = Type;
-
+	#pragma omp parallel for
 	for(sLong i=0; i<m_nRecords; i++)
 	{
 		CSG_Table_Record *pRecord = m_Records[i];
 
-		CSG_Table_Value  *pValue  = CSG_Table_Record::_Create_Value(Type);
+		bool bNoData = pRecord->is_NoData(Field);
 
-		(*pValue) = *pRecord->m_Values[iField];
+		CSG_Table_Value *pValue = pRecord->m_Values[Field];
 
-		delete(pRecord->m_Values[iField]);
+		pRecord->m_Values[Field] = CSG_Table_Record::_Create_Value(Type);
 
-		pRecord->m_Values[iField] = pValue;
+		if( bNoData )
+		{
+			if( Type != SG_DATATYPE_String && Type != SG_DATATYPE_Binary )
+			{
+				pRecord->m_Values[Field]->Set_Value(Get_NoData_Value());
+			}
+		}
+		else if( pValue->Get_Type() == SG_TABLE_VALUE_TYPE_String && SG_Data_Type_is_Numeric(Type) )
+		{
+			CSG_String String(pValue->asString()); double Value;
 
-		pRecord->Set_Modified();
+			if( !String.asDouble(Value) )
+			{
+				Value = Get_NoData_Value();
+			}
+
+			pRecord->m_Values[Field]->Set_Value(Value);
+		}
+		else
+		{
+			*(pRecord->m_Values[Field]) = *pValue;
+		}
+
+		delete(pValue);
+
+		pRecord->m_Flags |= SG_TABLE_REC_FLAG_Modified;
 	}
 
+	m_Field_Type[Field] = Type;
+
+	Set_Update_Flag(); _Stats_Invalidate(Field); Set_Modified();
+
 	return( true );
 }
 
 //---------------------------------------------------------
-int CSG_Table::Get_Field_Length(int iField, int Encoding)	const
+/**
+* Returns the maximum number of characters for data type string
+* or the number of bytes used for all other data types.
+*/
+//---------------------------------------------------------
+int CSG_Table::Get_Field_Length(int Field, int Encoding) const
 {
 	size_t Length = 0;
 
-	if( iField >= 0 && iField < m_nFields && m_Field_Type[iField] == SG_DATATYPE_String )
+	if( Field >= 0 && Field < m_nFields )
 	{
-		for(sLong i=0; i<m_nRecords; i++)
+		switch( m_Field_Type[Field] )
 		{
-			CSG_String s(m_Records[i]->asString(iField));
+		default:
+			Length = SG_Data_Type_Get_Size(m_Field_Type[Field]);
+			break;
 
-			size_t nBytes;
+		case SG_DATATYPE_Date:
+			Length = 10; // => YYYY-MM-DD
+			break;
 
-			switch( Encoding )
+		case SG_DATATYPE_String:
+			for(sLong i=0; i<m_nRecords; i++)
 			{
-			default                      :
-			case SG_FILE_ENCODING_UTF7   : nBytes = s.Length()            ; break;
-			case SG_FILE_ENCODING_UTF8   : nBytes = s.to_UTF8().Get_Size(); break;
-			case SG_FILE_ENCODING_UTF16LE:
-			case SG_FILE_ENCODING_UTF16BE: nBytes = s.Length() * 2        ; break;
-			case SG_FILE_ENCODING_UTF32LE:
-			case SG_FILE_ENCODING_UTF32BE: nBytes = s.Length() * 4        ; break;
-			}
+				CSG_String s(m_Records[i]->asString(Field));
 
-			if( Length < nBytes )
-			{
-				Length = nBytes;
+				size_t nBytes;
+
+				switch( Encoding )
+				{
+				default                      :
+				case SG_FILE_ENCODING_UTF7   : nBytes = s.Length()            ; break;
+				case SG_FILE_ENCODING_UTF8   : nBytes = s.to_UTF8().Get_Size(); break;
+				case SG_FILE_ENCODING_UTF16LE:
+				case SG_FILE_ENCODING_UTF16BE: nBytes = s.Length() * 2        ; break;
+				case SG_FILE_ENCODING_UTF32LE:
+				case SG_FILE_ENCODING_UTF32BE: nBytes = s.Length() * 4        ; break;
+				}
+
+				if( Length < nBytes )
+				{
+					Length = nBytes;
+				}
 			}
+			break;
 		}
 	}
 
@@ -694,11 +722,11 @@ int CSG_Table::Get_Field(const char       *Name) const { return( Get_Field(CSG_S
 int CSG_Table::Get_Field(const wchar_t    *Name) const { return( Get_Field(CSG_String(Name))); }
 int CSG_Table::Get_Field(const CSG_String &Name) const
 {
-	for(int iField=0; iField<Get_Field_Count(); iField++)
+	for(int Field=0; Field<Get_Field_Count(); Field++)
 	{
-		if( !Name.Cmp(Get_Field_Name(iField)) )
+		if( !Name.Cmp(Get_Field_Name(Field)) )
 		{
-			return( iField );
+			return( Field );
 		}
 	}
 
@@ -735,9 +763,9 @@ bool CSG_Table::Find_Field(const CSG_String &Name, int &Index) const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Records							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -939,7 +967,7 @@ bool CSG_Table::Set_Count(sLong nRecords)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1104,9 +1132,9 @@ CSG_Table_Record * CSG_Table::Find_Record(int iField, double Value, bool bCreate
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Value Access					 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1191,9 +1219,9 @@ bool CSG_Table::Get_Value(sLong iRecord, int iField, double      &Value) const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						Statistics						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1271,9 +1299,9 @@ bool CSG_Table::_Stats_Update(int iField) const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1299,9 +1327,47 @@ bool CSG_Table::On_Update(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
+//						Sort							 //
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Table::Sort(const char       *Field, bool bAscending) { return( Sort(Get_Field(Field), bAscending) ); }
+bool CSG_Table::Sort(const wchar_t    *Field, bool bAscending) { return( Sort(Get_Field(Field), bAscending) ); }
+bool CSG_Table::Sort(const CSG_String &Field, bool bAscending) { return( Sort(Get_Field(Field), bAscending) ); }
+bool CSG_Table::Sort(int               Field, bool bAscending)
+{
+	CSG_Index Index; return( Set_Index(Index, Field, bAscending) && Sort(Index) );
+}
+
+//---------------------------------------------------------
+bool CSG_Table::Sort(const CSG_Index &Index)
+{
+	if( Get_Count() > 0 && Get_Count() == Index.Get_Count() )
+	{
+		CSG_Table_Record **Records = m_Records; m_Records = (CSG_Table_Record **)SG_Malloc(m_nBuffer * sizeof(CSG_Table_Record *));
+
+		for(sLong i=0; i<Get_Count(); i++)
+		{
+			m_Records[i] = Records[Index[i]];
+		}
+
+		SG_Free(Records);
+
+		Del_Index();
+
+		return( true );
+	}
+
+	return( false );
+}
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
 //						Index							 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1344,21 +1410,25 @@ bool CSG_Table::Del_Index(void)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Toggle_Index(int iField)
+bool CSG_Table::Toggle_Index(int Field)
 {
-	if( iField < 0 || iField >= m_nFields )
+	if( Field < 0 || Field >= m_nFields )
 	{
 		return( false );
 	}
 
-	if( iField != Get_Index_Field(0) )
+	if( Field != Get_Index_Field(0) )
 	{
-		return( Set_Index(iField, TABLE_INDEX_Ascending) );
+		Del_Index();
+
+		return( Set_Index(Field, TABLE_INDEX_Ascending) );
 	}
 
 	if( Get_Index_Order(0) == TABLE_INDEX_Ascending )
 	{
-		return( Set_Index(iField, TABLE_INDEX_Descending) );
+		m_Index_Fields[0] = -m_Index_Fields[0]; m_Index.Invert();
+
+		return( true );
 	}
 
 	return( Del_Index() );
@@ -1366,7 +1436,7 @@ bool CSG_Table::Toggle_Index(int iField)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1394,20 +1464,16 @@ public:
 
 		switch( m_pTable->Get_Field_Type(m_Field) )
 		{
-		default: {
-			double d =
-				m_pTable->Get_Record(a)->asDouble(m_Field) -
-				m_pTable->Get_Record(b)->asDouble(m_Field);
-
-			return( d < 0. ? -1 : d > 0. ? 1 : 0 );
-		}
+		default: { double Value[2] = { 0., 0. };
+			m_pTable->Get_Value(a, m_Field, Value[0]);
+			m_pTable->Get_Value(b, m_Field, Value[1]);
+			return( Value[0] < Value[1] ? -1 : Value[0] > Value[1] ? 1 : 0 ); }
 
 		case SG_DATATYPE_String:
-		case SG_DATATYPE_Date  :
-			return( SG_STR_CMP(
-				m_pTable->Get_Record(a)->asString(m_Field),
-				m_pTable->Get_Record(b)->asString(m_Field))
-			);
+		case SG_DATATYPE_Date  : { CSG_String Value[2];
+			m_pTable->Get_Value(a, m_Field, Value[0]);
+			m_pTable->Get_Value(b, m_Field, Value[1]);
+			return( Value[0].Cmp(Value[1]) ); }
 		}
 	}
 
@@ -1432,7 +1498,7 @@ bool CSG_Table::Set_Index(CSG_Index &Index, int Field, bool bAscending) const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1440,7 +1506,7 @@ class CSG_Table_Record_Compare_Fields : public CSG_Index::CSG_Index_Compare
 {
 public:
 	CSG_Table_Record_Compare_Fields(const CSG_Table *pTable, int Fields[], int nFields, bool Ascending)
-		: m_pTable(pTable), m_Fields(Fields), m_nFields(nFields)
+		: m_Fields(Fields), m_nFields(nFields), m_pTable(pTable)
 	{
 		m_Ascending.Create(nFields);
 
@@ -1456,7 +1522,7 @@ public:
 	}
 
 	CSG_Table_Record_Compare_Fields(const CSG_Table *pTable, int Fields[], int nFields, int Ascending[])
-		: m_pTable(pTable), m_Fields(Fields), m_nFields(nFields)
+		: m_Fields(Fields), m_nFields(nFields), m_pTable(pTable)
 	{
 		m_Ascending.Create(nFields);
 
@@ -1475,28 +1541,29 @@ public:
 
 	virtual int			Compare		(const sLong _a, const sLong _b)
 	{
-		int	Difference	= 0;
+		int Difference = 0;
 
 		for(int i=0; !Difference && i<m_nFields; i++)
 		{
-			int	Field = m_Fields[i];
+			int Field = m_Fields[i];
 
 			sLong a = m_Ascending[i] ? _a : _b;
 			sLong b = m_Ascending[i] ? _b : _a;
 
 			switch( m_pTable->Get_Field_Type(Field) )
 			{
-			default: {
-				double d = m_pTable->Get_Record(a)->asDouble(Field);
-				d       -= m_pTable->Get_Record(b)->asDouble(Field);
-				Difference = d < 0. ? -1 : d > 0. ? 1 : 0;
-			}	break;
+			default: { double Value[2] = { 0., 0. };
+				m_pTable->Get_Value(a, Field, Value[0]);
+				m_pTable->Get_Value(b, Field, Value[1]);
+				Difference = Value[0] < Value[1] ? -1 : Value[0] > Value[1] ? 1 : 0;
+				break; }
 
 			case SG_DATATYPE_String:
-			case SG_DATATYPE_Date  :
-				CSG_String   s    (m_pTable->Get_Record(a)->asString(Field));
-				Difference = s.Cmp(m_pTable->Get_Record(b)->asString(Field));
-				break;
+			case SG_DATATYPE_Date  : { CSG_String Value[2];
+				m_pTable->Get_Value(a, Field, Value[0]);
+				m_pTable->Get_Value(b, Field, Value[1]);
+				Difference = Value[0].Cmp(Value[1]);
+				break; }
 			}
 		}
 
@@ -1554,9 +1621,9 @@ void CSG_Table::_Index_Update(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
