@@ -47,7 +47,6 @@
 
 //---------------------------------------------------------
 #include <saga_api/saga_api.h>
-#include <saga_api/nanoflann.hpp>
 
 #include <wx/notebook.h>
 #include <wx/html/htmlwin.h>
@@ -179,22 +178,22 @@ wxString CDLG_About::_Get_Version(void)
 	#if defined(COMPILER)
 		if( *COMPILER )
 		{
-			s	+= "<hr><i>Compiled with</i><br>[ " COMPILER " ]<br>";
+			s += "<hr><i>Compiled with</i><br>[ " COMPILER " ]<br>";
 
 			#if defined(_SAGA_MSW) && defined(_MSC_VER) && _MSC_VER >= 1900
-				s	+= "[ Microsoft Visual Studio "
+				s += "[ Microsoft Visual Studio ";
 				#if   _MSC_VER == 1900
-					"2015 (14)"
-				#elif _MSC_VER <= 1920
-					"2017 (15)"
-				#elif _MSC_VER <= 1930
-					"2019 (16)"
-				#elif _MSC_VER <= 1940
-					"2022 (17)"
+					s += "2015 (14)";
+				#elif _MSC_VER >= 1910 && _MSC_VER < 1920
+					s += "2017 (15)";
+				#elif _MSC_VER >= 1920 && _MSC_VER < 1930
+					s += wxString::Format("2019 (16.%d)", _MSC_VER - 1920);
+				#elif _MSC_VER >= 1930 && _MSC_VER < 1950
+					s += wxString::Format("2022 (17.%d)", _MSC_VER - 1930);
 				#else
-					"(unknown version)"
+					s += "(unknown version)";
 				#endif
-				" ]<br>";
+				s += " ]<br>";
 			#endif
 		}
 	#endif
@@ -229,10 +228,7 @@ wxString CDLG_About::_Get_Version(void)
 	s	+= "<i>and</i><br>";
 	s	+= "The Nearest Neighbor (NN) search with KD-trees library<br>";
 	s	+= "<a href=\"https://github.com/jlblancoc/nanoflann/\">";
-	s	+= wxString::Format("nanoflann %d.%d.%d",
-			(NANOFLANN_VERSION&0xf00)/0x100,
-			(NANOFLANN_VERSION&0x0f0)/0x010,
-			(NANOFLANN_VERSION&0x00f)/0x001);
+	s	+= CSG_KDTree::Get_Version();
 	s	+= "</a>";
 
 	//-----------------------------------------------------
