@@ -47,11 +47,8 @@
 
 //---------------------------------------------------------
 #include "res_commands.h"
-#include "res_controls.h"
 #include "res_dialogs.h"
 
-#include "saga_frame.h"
-
 #include "helper.h"
 
 #include "active.h"
@@ -125,25 +122,22 @@ CWKSP_Layer *	Get_Active_Layer(void)
 CWKSP_Layer::CWKSP_Layer(CSG_Data_Object *pObject)
 	: CWKSP_Data_Item(pObject)
 {
-	m_pClassify		= new CWKSP_Layer_Classify;
-	m_pLegend		= new CWKSP_Layer_Legend(this);
+	m_pClassify  = new CWKSP_Layer_Classify;
+	m_pLegend    = new CWKSP_Layer_Legend(this);
 
-	m_pHistogram	= NULL;
+	m_pHistogram = NULL;
 
-	m_Edit_Index	= 0;
+	m_Edit_Index = 0;
 }
 
 //---------------------------------------------------------
 CWKSP_Layer::~CWKSP_Layer(void)
 {
-	if( g_pSAGA_Frame ) { g_pSAGA_Frame->Freeze(); }
-
-	if( g_pMaps       ) { g_pMaps->Del(this);      }
-
-	if( m_pClassify   ) { delete(m_pClassify);     }
-	if( m_pLegend     ) { delete(m_pLegend  );     }
-
-	if( g_pSAGA_Frame ) { g_pSAGA_Frame->Thaw();   }
+	MDI_Freeze();
+	if( g_pMaps     ) { g_pMaps->Del(this);  }
+	if( m_pClassify ) { delete(m_pClassify); }
+	if( m_pLegend   ) { delete(m_pLegend  ); }
+	MDI_Thaw();
 }
 
 
@@ -156,18 +150,14 @@ bool CWKSP_Layer::On_Command(int Cmd_ID)
 {
 	switch( Cmd_ID )
 	{
-	default:
-		return( CWKSP_Data_Item::On_Command(Cmd_ID) );
+	default: return( CWKSP_Data_Item::On_Command(Cmd_ID) );
 
 	case ID_CMD_WKSP_ITEM_RETURN:
-	case ID_CMD_SHAPES_SHOW:
-	case ID_CMD_GRID_SHOW:
-	case ID_CMD_TIN_SHOW:
-	case ID_CMD_POINTCLOUD_SHOW:
+	case ID_CMD_DATA_SHOW_MAP   :
 		g_pMaps->Add(this);
 		break;
 
-	case ID_CMD_DATA_PROJECTION:
+	case ID_CMD_DATA_PROJECTION :
 		_Set_Projection();
 		break;
 	}
@@ -845,23 +835,28 @@ bool CWKSP_Layer::_Set_Thumbnail(bool bRefresh)
 {
 	if( m_pObject && m_Thumbnail.IsOk() && m_Thumbnail.GetWidth() > 0 && m_Thumbnail.GetHeight() > 0 )
 	{
-		wxMemoryDC		dc;
-		wxRect			r(0, 0, m_Thumbnail.GetWidth(), m_Thumbnail.GetHeight());
-		CWKSP_Map_DC	dc_Map(Get_Extent(), r, 1., SG_GET_RGB(255, 255, 255));
+		CSG_Map_DC dc_Map(Get_Extent(), m_Thumbnail.GetSize(), 1., Get_Color_asInt(SYS_Get_Color(wxSYS_COLOUR_WINDOW)));
 
-		Draw(dc_Map, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS|LAYER_DRAW_FLAG_THUMBNAIL);
+		if( Draw(dc_Map, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS|LAYER_DRAW_FLAG_THUMBNAIL) )
+		{
+			wxBitmap Bitmap;
 
-		dc.SelectObject(m_Thumbnail);
-		dc.SetBackground(*wxWHITE_BRUSH);
-		dc.Clear();
+			if( dc_Map.Get_Bitmap(Bitmap) )
+			{
+				wxMemoryDC dc;
 
-		dc_Map.Draw(dc);
+				dc.SelectObject(m_Thumbnail);
+				dc.DrawBitmap(Bitmap, 0, 0, false);
+				dc.SelectObject(wxNullBitmap);
 
-		dc.SelectObject(wxNullBitmap);
+				if( bRefresh && g_pData_Buttons )
+				{
+					g_pData_Buttons->Refresh(false);
+				}
 
-		if( bRefresh && g_pData_Buttons ) { g_pData_Buttons->Refresh(false); }
-
-		return( true );
+				return( true );
+			}
+		}
 	}
 
 	return( false );
@@ -910,37 +905,20 @@ CSG_Rect CWKSP_Layer::Get_Extent(void)
 //---------------------------------------------------------
 void CWKSP_Layer::_Set_Projection(void)
 {
-	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true);	// CCRS_Picker
+	CSG_Projection &CRS = Get_Object()->Get_Projection();
 
-	CSG_Projection	Projection(Get_Object()->Get_Projection());
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("pj_proj4", 15, true); // CCRS_Picker
 
 	if(	pTool
-	&&  pTool->Set_Parameter("CRS_EPSG"     , Projection.Get_Authority_ID())
-	&&  pTool->Set_Parameter("CRS_EPSG_AUTH", Projection.Get_Authority   ())
-	&&  pTool->Set_Parameter("CRS_PROJ4"    , Projection.Get_Proj4       ())
+	&&  pTool->Set_Parameter("CRS_WKT" , CRS.Get_WKT2())
+	&&  pTool->Set_Parameter("CRS_PROJ", CRS.Get_PROJ())
 	&&	pTool->On_Before_Execution() && DLG_Parameters(pTool->Get_Parameters()) )
 	{
-		Projection.Destroy();
+		CSG_Projection new_CRS(pTool->Get_Parameter("CRS_WKT")->asString(), pTool->Get_Parameter("CRS_PROJ")->asString());
 
-		if( pTool->Get_Parameter("CRS_EPSG")->asInt() > 0 )
+		if( new_CRS.is_Okay() && CRS.Get_WKT2().CmpNoCase(new_CRS.Get_WKT2()) )
 		{
-			Projection.Create(
-				pTool->Get_Parameter("CRS_EPSG"     )->asInt   (),
-				pTool->Get_Parameter("CRS_EPSG_AUTH")->asString()
-			);
-		}
-		
-		if( !Projection.is_Okay() )
-		{
-			Projection.Create(
-				pTool->Get_Parameter("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4
-			);
-		}
-
-		if( Projection.is_Okay() && !Projection.is_Equal(Get_Object()->Get_Projection()) )
-		{
-			Get_Object()->Get_Projection().Create(Projection);
-			Get_Object()->Set_Modified();
+			CRS.Create(new_CRS); Get_Object()->Set_Modified();
 
 			DataObject_Changed();
 		}
@@ -987,7 +965,8 @@ bool CWKSP_Layer::Set_Colors(CSG_Colors *pColors)
 //---------------------------------------------------------
 bool CWKSP_Layer::Set_Color_Range(double Minimum, double Maximum)
 {
-	m_Parameters.Set_Parameter("STRETCH_DEFAULT"  , 3);	// manual
+	m_Parameters.Set_Parameter("COLORS_TYPE"      , CLASSIFY_GRADUATED);
+	m_Parameters.Set_Parameter("STRETCH_DEFAULT"  , _TL("Manual"));	// manual
 	m_Parameters.Set_Parameter("METRIC_ZRANGE.MIN", Minimum < Maximum ? Minimum : Maximum);
 	m_Parameters.Set_Parameter("METRIC_ZRANGE.MAX", Minimum < Maximum ? Maximum : Minimum);
 
@@ -1037,25 +1016,25 @@ bool CWKSP_Layer::do_Show(CSG_Rect const &Map_Extent, bool bIntersects)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CWKSP_Layer::Draw(CWKSP_Map_DC &dc_Map, int Flags, CSG_Data_Object *pObject)
+bool CWKSP_Layer::Draw(CSG_Map_DC &dc_Map, int Flags, CSG_Data_Object *pObject)
 {
 	if( pObject && pObject->is_Valid() && pObject->Get_ObjectType() == m_pObject->Get_ObjectType() )
 	{
-		CSG_Data_Object	*pOriginal	= m_pObject;
+		CSG_Data_Object	*pOriginal = m_pObject;
 
-		m_pObject	= pObject;
+		m_pObject = pObject;
 
 		On_Draw(dc_Map, Flags);
 
-		m_pObject	= pOriginal;
+		m_pObject = pOriginal;
 	}
 	else
 	{
 		On_Draw(dc_Map, Flags);
 	}
 
-	dc_Map.dc.SetBrush(wxNullBrush);
-	dc_Map.dc.SetPen  (wxNullPen  );
+	dc_Map.SetBrush(wxNullBrush);
+	dc_Map.SetPen  (wxNullPen  );
 
 	return( true );
 }
@@ -1108,20 +1087,13 @@ bool CWKSP_Layer::Show(int Flags)
 	case SG_UI_DATAOBJECT_SHOW_MAP_LAST:
 		return( Show(g_pMaps->Get_Map(g_pMaps->Get_Count() - 1)) );
 
-	case SG_UI_DATAOBJECT_SHOW_MAP_ACTIVE: {
-		CVIEW_Map *pView = (CVIEW_Map *)g_pSAGA_Frame->Get_Active_Child(ID_VIEW_MAP);
-		return( Show(pView ? pView->Get_Map() : NULL) ); }
+	case SG_UI_DATAOBJECT_SHOW_MAP_ACTIVE:
+		return( Show(MDI_Get_Active_Map() ? ((CVIEW_Map *)MDI_Get_Active_Map())->Get_Map() : NULL) );
 	}
 
 	return( false );
 }
 
-//---------------------------------------------------------
-bool CWKSP_Layer::Update(CWKSP_Layer *pChanged)
-{
-	return( pChanged == this );
-}
-
 //---------------------------------------------------------
 void CWKSP_Layer::On_Update_Views(bool bAll)
 {
