@@ -92,6 +92,9 @@ CWKSP_PointCloud::CWKSP_PointCloud(CSG_PointCloud *pPointCloud)
 
 	CSG_Table &LUT = *m_Parameters("LUT")->asTable(); LUT.Del_Records();
 
+	LUT.Set_Field_Type(LUT_MIN, SG_DATATYPE_String);
+	LUT.Set_Field_Type(LUT_MAX, SG_DATATYPE_String);
+
 	#define ADD_CLASS(color, name, value) { CSG_Table_Record &r = *LUT.Add_Record(); r.Set_Value(0, color); r.Set_Value(1, name); r.Set_Value(3, value); r.Set_Value(4, value); }
 
 	ADD_CLASS(12632256, "Created, Never Classified"   ,  0);
@@ -178,10 +181,15 @@ wxString CWKSP_PointCloud::Get_Description(void)
 //---------------------------------------------------------
 wxMenu * CWKSP_PointCloud::Get_Menu(void)
 {
-	wxMenu *pMenu = new wxMenu(m_pObject->Get_Name());
+	wxMenu *pSubMenu, *pMenu = new wxMenu(m_pObject->Get_Name());
 
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_POINTCLOUD_SHOW);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SHOW_MAP);
+	if( MDI_Get_Active_Map() )
+	{
+		CMD_Menu_Add_Item(pMenu, false, ID_CMD_MAP_ZOOM_ACTIVE);
+		CMD_Menu_Add_Item(pMenu, false, ID_CMD_MAP_PAN_ACTIVE);
+	}
 
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVE);
@@ -200,32 +208,37 @@ wxMenu * CWKSP_PointCloud::Get_Menu(void)
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_METADATA);
 
 	pMenu->AppendSeparator();
-	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_SHAPES_HISTOGRAM);
-
-	pMenu->AppendSeparator();
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_SET_LUT);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_CLASSIFY);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SETTINGS_COPY);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_FORCE_UPDATE);
 
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Color Stretch"), pSubMenu = new wxMenu());
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_RANGE_MINMAX   );
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_RANGE_STDDEV150);
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_RANGE_STDDEV200);
+
+	//-----------------------------------------------------
 	pMenu->AppendSeparator();
 
-	//-----------------------------------------------------
-	wxMenu *pSubMenu = new wxMenu(_TL("Classification"));
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Attributes"), pSubMenu = new wxMenu());
+	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_TABLE_SHOW);
+//	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_SHAPES_SAVE_ATTRIBUTES);
 
-	CMD_Menu_Add_Item(pSubMenu	, false, ID_CMD_POINTCLOUD_RANGE_MINMAX);
-	CMD_Menu_Add_Item(pSubMenu	, false, ID_CMD_POINTCLOUD_RANGE_STDDEV150);
-	CMD_Menu_Add_Item(pSubMenu	, false, ID_CMD_POINTCLOUD_RANGE_STDDEV200);
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Charts"    ), pSubMenu = new wxMenu());
+	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_DATA_HISTOGRAM);
+//	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SCATTERPLOT);
+//	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_DATA_DIAGRAM);
 
-	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Classification"), pSubMenu);
-
-	//-----------------------------------------------------
-	wxMenu *pTable = new wxMenu(_TL("Table"));
-
-	CMD_Menu_Add_Item(pTable,  true, ID_CMD_TABLE_SHOW);
-//	CMD_Menu_Add_Item(pTable,  true, ID_CMD_TABLE_DIAGRAM);
-//	CMD_Menu_Add_Item(pTable, false, ID_CMD_TABLE_SCATTERPLOT);
-
-	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Attributes"), pTable);
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Selection" ), pSubMenu = new wxMenu());
+	if( MDI_Get_Active_Map() && Get_PointCloud()->Get_Selection_Count() > 0 )
+	{
+		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_ZOOM_SELECTION);
+		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_PAN_SELECTION);
+	}
+	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_DATA_SELECTION_CLEAR);
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SELECTION_INVERT);
+	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_TABLE_SELECT_NUMERIC);
+	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_TABLE_SELECT_STRING);
 
 	return( pMenu );
 }
@@ -246,41 +259,50 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
 	case ID_CMD_POINTCLOUD_LAST:
 		break;
 
-	case ID_CMD_SHAPES_SET_LUT:
+	case ID_CMD_DATA_CLASSIFY:
 		_LUT_Create();
 		break;
 
-	case ID_CMD_POINTCLOUD_RANGE_MINMAX:
+	case ID_CMD_DATA_RANGE_MINMAX:
 		Set_Color_Range(
 			Get_PointCloud()->Get_Minimum(m_fValue),
 			Get_PointCloud()->Get_Maximum(m_fValue)
 		);
 		break;
 
-	case ID_CMD_POINTCLOUD_RANGE_STDDEV150:
+	case ID_CMD_DATA_RANGE_STDDEV150:
 		Set_Color_Range(
 			Get_PointCloud()->Get_Mean(m_fValue) - 1.5 * Get_PointCloud()->Get_StdDev(m_fValue),
 			Get_PointCloud()->Get_Mean(m_fValue) + 1.5 * Get_PointCloud()->Get_StdDev(m_fValue)
 		);
 		break;
 
-	case ID_CMD_POINTCLOUD_RANGE_STDDEV200:
+	case ID_CMD_DATA_RANGE_STDDEV200:
 		Set_Color_Range(
 			Get_PointCloud()->Get_Mean(m_fValue) - 2. * Get_PointCloud()->Get_StdDev(m_fValue),
 			Get_PointCloud()->Get_Mean(m_fValue) + 2. * Get_PointCloud()->Get_StdDev(m_fValue)
 		);
 		break;
 
-	case ID_CMD_SHAPES_EDIT_SEL_CLEAR:
-		Get_PointCloud()->Select();
-		Update_Views();
-		break;
+	//-----------------------------------------------------
+	case ID_CMD_DATA_SELECTION_CLEAR : Get_PointCloud()->Select       (); Update_Views(); break;
+	case ID_CMD_DATA_SELECTION_INVERT: Get_PointCloud()->Inv_Selection(); Update_Views(); break;
 
-	case ID_CMD_SHAPES_EDIT_SEL_INVERT:
-		Get_PointCloud()->Inv_Selection();
-		Update_Views();
-		break;
+	case ID_CMD_TABLE_SELECT_NUMERIC : { CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool("shapes_tools", 3);
+			if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
+			{
+				pTool->Execute();
+			}
+ 		break; }
 
+	case ID_CMD_TABLE_SELECT_STRING  : { CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool("shapes_tools", 4);
+			if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
+			{
+				pTool->Execute();
+			}
+ 		break; }
+
+	//-----------------------------------------------------
 	case ID_CMD_SHAPES_EDIT_DEL_SHAPE:
 		if( Get_PointCloud()->Get_Selection_Count() > 0 && DLG_Message_Confirm(_TL("Delete selected point(s)."), _TL("Edit Point Cloud")) )
 		{
@@ -290,7 +312,7 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
 		}
 		break;
 
-    case ID_CMD_SHAPES_EDIT_SEL_COPY_TO_NEW_LAYER:
+    case ID_CMD_SHAPES_EDIT_SEL_COPY:
         if( Get_PointCloud()->Get_Selection_Count() > 0 )
         {
             CSG_PointCloud *pCopy = new CSG_PointCloud(Get_PointCloud());
@@ -308,17 +330,10 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
         }
         break;
 
-	case ID_CMD_TABLE_SHOW:
-		m_pTable->Toggle_View();
-		break;
-
-	case ID_CMD_TABLE_DIAGRAM:
-		m_pTable->Toggle_Diagram();
-		break;
-
-	case ID_CMD_SHAPES_HISTOGRAM:
-		Histogram_Toggle();
-		break;
+	//-----------------------------------------------------
+	case ID_CMD_TABLE_SHOW    : m_pTable->Toggle_View   (); break;
+	case ID_CMD_DATA_DIAGRAM  : m_pTable->Toggle_Diagram(); break;
+	case ID_CMD_DATA_HISTOGRAM: Histogram_Toggle        (); break;
 	}
 
 	return( true );
@@ -332,11 +347,11 @@ bool CWKSP_PointCloud::On_Command_UI(wxUpdateUIEvent &event)
 	default:
 		return( CWKSP_Layer::On_Command_UI(event) );
 
-    case ID_CMD_SHAPES_EDIT_SEL_CLEAR:
+    case ID_CMD_DATA_SELECTION_CLEAR:
         event.Enable(Get_PointCloud()->Get_Selection_Count() > 0);
         break;
 
-    case ID_CMD_SHAPES_EDIT_SEL_INVERT:
+    case ID_CMD_DATA_SELECTION_INVERT:
         event.Enable(true);
         break;
 
@@ -344,24 +359,16 @@ bool CWKSP_PointCloud::On_Command_UI(wxUpdateUIEvent &event)
         event.Enable(Get_PointCloud()->Get_Selection_Count() > 0);
         break;
 
-    case ID_CMD_SHAPES_EDIT_SEL_COPY_TO_NEW_LAYER:
+    case ID_CMD_SHAPES_EDIT_SEL_COPY:
         event.Enable(Get_PointCloud()->Get_Selection_Count() > 0);
         break;
 
 	case ID_CMD_POINTCLOUD_LAST:
 		break;
 
-	case ID_CMD_TABLE_SHOW:
-		event.Check(m_pTable->Get_View() != NULL);
-		break;
-
-	case ID_CMD_TABLE_DIAGRAM:
-		event.Check(m_pTable->Get_Diagram() != NULL);
-		break;
-
-	case ID_CMD_SHAPES_HISTOGRAM:
-		event.Check(m_pHistogram != NULL);
-		break;
+	case ID_CMD_TABLE_SHOW    : event.Check(m_pTable->Get_View   () != NULL); break;
+	case ID_CMD_DATA_DIAGRAM  : event.Check(m_pTable->Get_Diagram() != NULL); break;
+	case ID_CMD_DATA_HISTOGRAM: event.Check(m_pHistogram            != NULL); break;
 	}
 
 	return( true );
@@ -466,14 +473,14 @@ void CWKSP_PointCloud::On_Parameters_Changed(void)
 
 		m_pClassify->Set_Mode(CLASSIFY_SINGLE);
 	}
-	else if( m_Parameters("COLORS_TYPE")->asInt() == CLASSIFY_OVERLAY )
+	else if( m_Parameters("COLORS_TYPE")->asInt() == 4 ) // CLASSIFY_RGB
 	{
 		m_pClassify->Set_Mode(CLASSIFY_RGB);
 	}
 
+	//-----------------------------------------------------
 	m_pObject->Set_Max_Samples(Get_PointCloud()->Get_Count() * (m_Parameters("MAX_SAMPLES")->asDouble() / 100.) );
 
-	//-----------------------------------------------------
 	long DefColor = m_Parameters("SINGLE_COLOR")->asColor();
 	m_Color_Pen   = wxColour(SG_GET_R(DefColor), SG_GET_G(DefColor), SG_GET_B(DefColor));
 
@@ -506,6 +513,20 @@ int CWKSP_PointCloud::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Para
 
 			pParameters->Get_Parameter("METRIC_ZRANGE")->asRange()->Set_Range(min, max);
 		}
+
+		if(	pParameter->Cmp_Identifier("LUT_ATTRIB") || (pParameter->Cmp_Identifier("COLORS_TYPE") && pParameter->asInt() == 1) ) // CLASSIFY_LUT
+		{
+			int Field = (*pParameters)("LUT_ATTRIB")->asInt();
+
+			if(	Field >= 0 && Field < Get_PointCloud()->Get_Field_Count() )
+			{
+				TSG_Data_Type Type = SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field))
+					? SG_DATATYPE_Double : SG_DATATYPE_String;
+		
+				(*pParameters)("LUT")->asTable()->Set_Field_Type(LUT_MIN, Type);
+				(*pParameters)("LUT")->asTable()->Set_Field_Type(LUT_MAX, Type);
+			}
+		}
 	}
 
 	//-----------------------------------------------------
@@ -600,11 +621,11 @@ void CWKSP_PointCloud::_LUT_Create(void)
 	}
 
 	{
-		CSG_String	Fields;
+		CSG_String Fields;
 
 		for(int i=0; i<Get_PointCloud()->Get_Field_Count(); i++)
 		{
-			Fields	+= CSG_String(Get_PointCloud()->Get_Field_Name(i)) + "|";
+			Fields += CSG_String(Get_PointCloud()->Get_Field_Name(i)) + "|";
 		}
 
 		Parameters("FIELD")->asChoice()->Set_Items(Fields);
@@ -618,45 +639,47 @@ void CWKSP_PointCloud::_LUT_Create(void)
 	//-----------------------------------------------------
 	DataObject_Changed();
 
-	int	Field	= Parameters("FIELD")->asInt();
+	int Field = Parameters("FIELD")->asInt();
 
-	CSG_Colors	Colors(*Parameters("COLOR")->asColors());
+	CSG_Colors Colors(*Parameters("COLOR")->asColors());
 
-	CSG_Table	Classes(m_Parameters("LUT")->asTable());
+	CSG_Table Classes(m_Parameters("LUT")->asTable());
 
 	switch( SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field)) ? Parameters("METHOD")->asInt() : 0 )
 	{
 	//-----------------------------------------------------
 	case 0:	// unique values
 		{
-			TSG_Data_Type	Type	= SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field))
-									? SG_DATATYPE_Double : SG_DATATYPE_String;
+			TSG_Data_Type Type = SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field))
+				? SG_DATATYPE_Double
+				: SG_DATATYPE_String;
 
 			Classes.Set_Field_Type(LUT_MIN, Type);
 			Classes.Set_Field_Type(LUT_MAX, Type);
 
-			CSG_Unique_String_Statistics	s;
+			CSG_Unique_String_Statistics Values; CSG_String Value;
 
 			#define MAX_CLASSES	1024
 
-			for(sLong iShape=0; iShape<Get_PointCloud()->Get_Count() && s.Get_Count()<MAX_CLASSES; iShape++)
+			for(sLong iPoint=0; iPoint<Get_PointCloud()->Get_Count() && Values.Get_Count()<MAX_CLASSES; iPoint++)
 			{
-				s	+= Get_PointCloud()->Get_Record(iShape)->asString(Field);
+				if( Get_PointCloud()->Get_Value(iPoint, Field, Value) )
+				{
+					Values += Value;
+				}
 			}
 
-			Colors.Set_Count(s.Get_Count());
+			Colors.Set_Count(Values.Get_Count());
 
-			for(int iClass=0; iClass<s.Get_Count(); iClass++)
+			for(int iClass=0; iClass<Values.Get_Count(); iClass++)
 			{
-				CSG_String	Value	= s.Get_Value(iClass);
+				CSG_Table_Record &Class = *Classes.Add_Record(); Value = Values.Get_Value(iClass);
 
-				CSG_Table_Record	*pClass	= Classes.Add_Record();
-
-				pClass->Set_Value(0, Colors[iClass]);	// Color
-				pClass->Set_Value(1, Value         );	// Name
-				pClass->Set_Value(2, Value         );	// Description
-				pClass->Set_Value(3, Value         );	// Minimum
-				pClass->Set_Value(4, Value         );	// Maximum
+				Class.Set_Value(0, Colors[iClass]); // Color
+				Class.Set_Value(1, Value         ); // Name
+				Class.Set_Value(2, ""            ); // Description
+				Class.Set_Value(3, Value         ); // Minimum
+				Class.Set_Value(4, Value         ); // Maximum
 			}
 		}
 		break;
@@ -664,28 +687,25 @@ void CWKSP_PointCloud::_LUT_Create(void)
 	//-----------------------------------------------------
 	case 1:	// equal intervals
 		{
-			double	Minimum, Maximum, Interval;
-
-			Interval	= Get_PointCloud()->Get_Range  (Field) / (double)Colors.Get_Count();
-			Minimum		= Get_PointCloud()->Get_Minimum(Field);
+			double Interval = Get_PointCloud()->Get_Range  (Field) / (double)Colors.Get_Count();
+			double Minimum  = Get_PointCloud()->Get_Minimum(Field);
 
 			Classes.Set_Field_Type(LUT_MIN, SG_DATATYPE_Double);
 			Classes.Set_Field_Type(LUT_MAX, SG_DATATYPE_Double);
 
 			for(int iClass=0; iClass<Colors.Get_Count(); iClass++, Minimum+=Interval)
 			{
-				Maximum	= iClass < Colors.Get_Count() - 1 ? Minimum + Interval : Get_PointCloud()->Get_Maximum(Field) + 1.;
+				CSG_Table_Record &Class = *Classes.Add_Record();
 
-				CSG_String	Name	= SG_Get_String(Minimum, -2)
-							+ " - " + SG_Get_String(Maximum, -2);
+				double Maximum = iClass < Colors.Get_Count() - 1 ? Minimum + Interval : Get_PointCloud()->Get_Maximum(Field) + 1.;
 
-				CSG_Table_Record	*pClass	= Classes.Add_Record();
+				CSG_String Name(SG_Get_String(Minimum, -2) + " - " + SG_Get_String(Maximum, -2));
 
-				pClass->Set_Value(0, Colors[iClass]);	// Color
-				pClass->Set_Value(1, Name          );	// Name
-				pClass->Set_Value(2, Name          );	// Description
-				pClass->Set_Value(3, Minimum       );	// Minimum
-				pClass->Set_Value(4, Maximum       );	// Maximum
+				Class.Set_Value(0, Colors[iClass]); // Color
+				Class.Set_Value(1, Name          ); // Name
+				Class.Set_Value(2, ""            ); // Description
+				Class.Set_Value(3, Minimum       ); // Minimum
+				Class.Set_Value(4, Maximum       ); // Maximum
 			}
 		}
 		break;
@@ -693,10 +713,7 @@ void CWKSP_PointCloud::_LUT_Create(void)
 	//-----------------------------------------------------
 	case 2:	// quantiles
 		{
-			TSG_Table_Index_Order	old_Order	= Get_PointCloud()->Get_Index_Order(0);
-			int						old_Field	= Get_PointCloud()->Get_Index_Field(0);
-
-			Get_PointCloud()->Set_Index(Field, TABLE_INDEX_Ascending);
+			CSG_Index Index; Get_PointCloud()->Set_Index(Index, Field, true);
 
 			Classes.Set_Field_Type(LUT_MIN, SG_DATATYPE_Double);
 			Classes.Set_Field_Type(LUT_MAX, SG_DATATYPE_Double);
@@ -706,36 +723,35 @@ void CWKSP_PointCloud::_LUT_Create(void)
 				Colors.Set_Count(Get_PointCloud()->Get_Count());
 			}
 
-			double	Minimum, Maximum, Count, iRecord;
+			double Maximum = Get_PointCloud()->Get_Minimum(Field);
 
-			Maximum	= Get_PointCloud()->Get_Minimum(Field);
-			iRecord	= Count	= Get_PointCloud()->Get_Count() / (double)Colors.Get_Count();
+			double Step	= Get_PointCloud()->Get_Count() / (double)Colors.Get_Count();
 
-			for(int iClass=0; iClass<Colors.Get_Count(); iClass++, iRecord+=Count)
+			for(int iClass=0; iClass<Colors.Get_Count(); iClass++)
 			{
-				Minimum	= Maximum;
-				Maximum	= iRecord < Get_PointCloud()->Get_Count() ? Get_PointCloud()->Get_Record_byIndex((int)iRecord)->asDouble(Field) : Get_PointCloud()->Get_Maximum(Field) + 1.;
+				CSG_Table_Record &Class = *Classes.Add_Record();
 
-				CSG_String	Name	= SG_Get_String(Minimum, -2)
-							+ " - " + SG_Get_String(Maximum, -2);
+				sLong i = (sLong)(Step * (1 + iClass));
 
-				CSG_Table_Record	*pClass	= Classes.Add_Record();
+				double Minimum = Maximum; Maximum = i < Get_PointCloud()->Get_Count()
+					? Get_PointCloud()->Get_Value(Index[i], Field)
+					: Get_PointCloud()->Get_Maximum(Field) + 1.;
 
-				pClass->Set_Value(0, Colors[iClass]);	// Color
-				pClass->Set_Value(1, Name          );	// Name
-				pClass->Set_Value(2, Name          );	// Description
-				pClass->Set_Value(3, Minimum       );	// Minimum
-				pClass->Set_Value(4, Maximum       );	// Maximum
+				CSG_String Name(SG_Get_String(Minimum, -2) + " - " + SG_Get_String(Maximum, -2));
+
+				Class.Set_Value(0, Colors[iClass]); // Color
+				Class.Set_Value(1, Name          ); // Name
+				Class.Set_Value(2, ""            ); // Description
+				Class.Set_Value(3, Minimum       ); // Minimum
+				Class.Set_Value(4, Maximum       ); // Maximum
 			}
-
-			Get_PointCloud()->Set_Index(old_Field, old_Order);
 		}
 		break;
 
 	//-----------------------------------------------------
 	case 3:	// natural breaks
 		{
-			CSG_Natural_Breaks	Breaks(Get_PointCloud(), Field, Colors.Get_Count(), Get_PointCloud()->Get_Count() > 4096 ? 256 : 0);
+			CSG_Natural_Breaks Breaks(Get_PointCloud(), Field, Colors.Get_Count(), Get_PointCloud()->Get_Count() > 4096 ? 256 : 0);
 
 			if( Breaks.Get_Count() <= Colors.Get_Count() ) return;
 
@@ -744,19 +760,18 @@ void CWKSP_PointCloud::_LUT_Create(void)
 
 			for(int iClass=0; iClass<Colors.Get_Count(); iClass++)
 			{
-				CSG_Table_Record	*pClass	= Classes.Add_Record();
+				CSG_Table_Record &Class = *Classes.Add_Record();
 
-				double	Minimum	= Breaks[iClass    ];
-				double	Maximum	= Breaks[iClass + 1];
+				double Minimum = Breaks[iClass    ];
+				double Maximum = Breaks[iClass + 1];
 
-				CSG_String	Name	= SG_Get_String(Minimum, -2)
-							+ " - " + SG_Get_String(Maximum, -2);
+				CSG_String Name(SG_Get_String(Minimum, -2) + " - " + SG_Get_String(Maximum, -2));
 
-				pClass->Set_Value(0, Colors[iClass]);	// Color
-				pClass->Set_Value(1, Name          );	// Name
-				pClass->Set_Value(2, Name          );	// Description
-				pClass->Set_Value(3, Minimum       );	// Minimum
-				pClass->Set_Value(4, Maximum       );	// Maximum
+				Class.Set_Value(0, Colors[iClass]); // Color
+				Class.Set_Value(1, Name          ); // Name
+				Class.Set_Value(2, ""            ); // Description
+				Class.Set_Value(3, Minimum       ); // Minimum
+				Class.Set_Value(4, Maximum       ); // Maximum
 			}
 		}
 		break;
@@ -767,7 +782,7 @@ void CWKSP_PointCloud::_LUT_Create(void)
 	{
 		m_Parameters("LUT")->asTable()->Assign(&Classes);
 
-		m_Parameters("COLORS_TYPE")->Set_Value(CLASSIFY_LUT);	// Lookup Table
+		m_Parameters("COLORS_TYPE")->Set_Value(CLASSIFY_LUT); // Lookup Table
 		m_Parameters("LUT_ATTRIB" )->Set_Value(Field);
 
 		Parameters_Changed();
@@ -782,32 +797,40 @@ void CWKSP_PointCloud::_LUT_Create(void)
 //---------------------------------------------------------
 wxString CWKSP_PointCloud::Get_Value(CSG_Point ptWorld, double Epsilon)
 {
-	CSG_Shape *pShape = Get_PointCloud()->Get_Shape(ptWorld, Epsilon);
+	wxString Value; sLong Index = Get_PointCloud()->Get_Point(ptWorld, Epsilon);
 
-	if( pShape != NULL )
+	if( Index >= 0 )
 	{
-		if( m_fValue >= 0 )
+		switch( m_fValue < 0 ? -1 : m_pClassify->Get_Mode() )
 		{
-			switch( m_pClassify->Get_Mode() )
+		case -1               :
+			Value.Printf("%s: %lld", _TL("Index"), Index + 1);
+			break;
+
+		default               :
+			Value.Printf("%f", Get_PointCloud()->Get_Value(Index, m_fValue));
+			break;
+
+		case CLASSIFY_RGB     : { int color = (int)Get_PointCloud()->Get_Value(Index, m_fValue);
+			Value.Printf("R%03d G%03d B%03d", SG_GET_R(color), SG_GET_G(color), SG_GET_B(color));
+			break; }
+
+		case CLASSIFY_LUT     :
+			if( SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(m_fValue)) )
 			{
-			case CLASSIFY_LUT     :
-				return( m_pClassify->Get_Class_Name_byValue(pShape->asDouble(m_fValue)) );
-
-			case CLASSIFY_DISCRETE:	default:
-				return( pShape->asString(m_fValue) );
-
-			case CLASSIFY_RGB     :
-				double Value = pShape->asDouble(m_fValue);
-				return( wxString::Format("R%03d G%03d B%03d", SG_GET_R((int)Value), SG_GET_G((int)Value), SG_GET_B((int)Value)) );
+				Value = m_pClassify->Get_Class_Name_byValue(Get_PointCloud()->Get_Value(Index, m_fValue));
 			}
-		}
-		else
-		{
-			return( wxString::Format("%s: %lld", _TL("Index"), pShape->Get_Index() + 1) );
+			else
+			{
+				CSG_String s; Get_PointCloud()->Get_Value(Index, m_fValue, s);
+
+				Value = m_pClassify->Get_Class_Name_byValue(s.c_str());
+			}
+			break;
 		}
 	}
 
-	return( _TL("") );
+	return( Value );
 }
 
 
@@ -844,9 +867,9 @@ wxMenu * CWKSP_PointCloud::Edit_Get_Menu(void)
 	wxMenu *pMenu = new wxMenu;
 
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_DEL_SHAPE);
-    CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_SEL_COPY_TO_NEW_LAYER);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_SEL_CLEAR);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_SEL_INVERT);
+    CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_SEL_COPY );
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SELECTION_CLEAR );
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SELECTION_INVERT);
 
 	return( pMenu );
 }
@@ -1092,7 +1115,16 @@ void CWKSP_PointCloud::_Draw_Points(CSG_Map_DC &dc_Map)
 				{
 					int Color;
 					
-					if( m_pClassify->Get_Class_Color_byValue(pPoints->Get_Value(m_fValue), Color) )
+					if( m_pClassify->Get_Mode() == CLASSIFY_LUT && !SG_Data_Type_is_Numeric(pPoints->Get_Field_Type(m_fValue)) )
+					{
+						CSG_String Value;
+
+						if( pPoints->Get_Value(m_fValue, Value) && m_pClassify->Get_Class_Color_byValue(Value, Color) )
+						{
+							_Draw_Point(dc_Map, x, y, Point.z, Color, m_PointSize);
+						}
+					}
+					else if( m_pClassify->Get_Class_Color_byValue(pPoints->Get_Value(m_fValue), Color) )
 					{
 						_Draw_Point(dc_Map, x, y, Point.z, Color, m_PointSize);
 					}
