@@ -68,7 +68,7 @@ CGeoref_Grid::CGeoref_Grid(bool bList)
 	Set_Description	(_TW(
 		"Georeferencing and rectification for grids. Either choose the attribute fields (x/y) "
 		"with the projected coordinates for the reference points (origin) or supply an "
-		"additional points layer with correspondend points in the target projection. "
+		"additional points layer with correspondent points in the target projection. "
 	));
 
 	//-----------------------------------------------------
@@ -402,7 +402,14 @@ CSG_Data_Object * CGeoref_Grid::Get_Target(CSG_Data_Object *pSource, TSG_Data_Ty
 			Type = pSource->asGrid()->Get_Type();
 		}
 
-		pTarget = m_Grid_Target.Get_Grid("TARGET_GRID", Type != SG_DATATYPE_Undefined ? Type : pSource->asGrid()->Get_Type());
+		if( m_bList )
+		{
+			pTarget = SG_Create_Grid(m_Grid_Target.Get_System(), Type);
+		}
+		else
+		{
+			pTarget = m_Grid_Target.Get_Grid("TARGET_GRID", Type);
+		}
 
 		if( pTarget )
 		{
